'use strict';
/*jshint asi: true */

var test = require('tap').test
  , transform = require('../../lib/transform')

function inspect(obj, depth) {
  console.log(require('util').inspect(obj, false, depth || 5, true));
}

function check(md, anchors, mode) {
  test('transforming ' + md , function (t) {
    var res = transform(md, mode)

    // remove wrapper
    var data = res.data.split('\n');

    // rig our expected value to include the wrapper
    var startLines = transform.start.split('\n')
      , anchorLines = anchors.split('\n')
      , endLines = transform.end.split('\n')
      , mdLines = md.split('\n')

    var rig = startLines
      .concat(anchorLines.slice(0, -2))
      .concat(endLines)
      .concat('')
      .concat(mdLines);

    t.ok(res.transformed, 'transforms it');
    t.deepEqual(data, rig, 'generates correct anchors')
    t.end()
  })
}

check(
    [ '# My Module'
    , 'Some text here'
    , '## API'
    , '### Method One'
    , 'works like this'
    , '### Method Two'
    , '#### Main Usage'
    , 'some main usage here'
    ].join('\n')
  , [ '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*\n\n'
    , '- [My Module](#my-module)\n'
    ,   '\t- [API](#api)\n'
    ,     '\t\t- [Method One](#method-one)\n'
    ,     '\t\t- [Method Two](#method-two)\n'
    ,         '\t\t\t- [Main Usage](#main-usage)\n\n\n'
    ].join('')
)

check(
    [ '# My Module using \\r\\n line endings'
    , 'Some text here'
    , '## API'
    , '### Method One'
    , 'works like this'
    , '### Method Two'
    , '#### Main Usage'
    , 'some main usage here'
    ].join('\r\n')
  , [ '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*\n\n'
    , '- [My Module using \\r\\n line endings](#my-module-using-\\r\\n-line-endings)\n'
    ,   '\t- [API](#api)\n'
    ,     '\t\t- [Method One](#method-one)\n'
    ,     '\t\t- [Method Two](#method-two)\n'
    ,         '\t\t\t- [Main Usage](#main-usage)\n\n\n'
    ].join('')
)

check(
    [ 'My Module'
    , '========='
    , 'Some text here'
    , 'API'
    , '---------'
    ].join('\n')
  , [ '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*\n\n'
    , '- [My Module](#my-module)\n'
    ,   '\t- [API](#api)\n\n\n'
    ].join('')
)

check(
    [ '# My Module #'
    , 'Some text here'
    , '## API ##'
    ].join('\n')
  , [ '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*\n\n'
    , '- [My Module](#my-module)\n'
    ,   '\t- [API](#api)\n\n\n'
    ].join('')
)

check(
    [ '## Title should be included'
    , ''
    , '```js'
    , 'function foo () {'
    , '  // ## This title should be ignored'
    , '}'
    , '## This title should also be ignored'
    , '```'
    , ''
    , '## Title should also be included'
    ].join('\n')
  , [ '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*\n\n'
    , '- [Title should be included](#title-should-be-included)\n'
    , '- [Title should also be included](#title-should-also-be-included)\n\n\n'
    ].join('')
)

check(
    [ '# Repeating A Title'
    , ''
    , '# Repeating A Title'
    ].join('\n')
  , [ '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*\n\n'
    , '- [Repeating A Title](#repeating-a-title)\n'
    , '- [Repeating A Title](#repeating-a-title-1)\n\n\n'
    ].join('')
)

check(
    [ '## Header'
    , 'some content'
    , '-- preceded by two dashes but has content, therefore "some content" should not be header'
    ].join('\n')
  , [ '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*\n\n',
      '- [Header](#header)\n\n\n',
    ].join('')
)

check(
    [ '# Different Kinds'
    , ''
    , 'In the Right Order'
    , '=================='
    ].join('\n')
  , [ '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*\n\n'
    , '- [Different Kinds](#different-kinds)\n'
    , '- [In the Right Order](#in-the-right-order)\n\n\n'
    ].join('')
)

check(
    [ 'Different Kinds 2'
    , '==============='
    , ''
    , '# In the Right Order 2'
    ].join('\n')
  , [ '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*\n\n'
    , '- [Different Kinds 2](#different-kinds-2)\n'
    , '- [In the Right Order 2](#in-the-right-order-2)\n\n\n'
    ].join('')
)

test('transforming when old toc exists', function (t) {
  var md = [ 
      '## Header'
    , 'some content'
    , ''
    , '<!-- START doctoc generated TOC please keep comment here to allow auto update -->'
    , '<!-- DON\'T EDIT THIS SECTION INSTEAD RE-RUN doctoc TO UPDATE -->'
    , '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*'
    , ''
    , '- [OldHeader](#oldheader)'
    , ''
    , '<!-- END doctoc generated TOC please keep comment here to allow auto update -->' 
    ].join('\n')

  var res = transform(md)

  t.ok(res.transformed, 'transforms it')     
  t.deepEqual(
      res.toc.split('\n')
    , [ '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*',
      '',
      '- [Header](#header)',
      '' ]
    , 'replaces old toc' 
  )
  
  t.deepEqual(
      res.wrappedToc.split('\n')
    , [ '<!-- START doctoc generated TOC please keep comment here to allow auto update -->',
        '<!-- DON\'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->',
        '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*',
        '',
        '- [Header](#header)',
        '',
        '<!-- END doctoc generated TOC please keep comment here to allow auto update -->' 
      ]
    , 'wraps old toc'
  )

  t.deepEqual(
      res.data.split('\n')
    , [ '## Header',
        'some content',
        '',
        '<!-- START doctoc generated TOC please keep comment here to allow auto update -->',
        '<!-- DON\'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->',
        '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*',
        '',
        '- [Header](#header)',
        '',
        '<!-- END doctoc generated TOC please keep comment here to allow auto update -->' ]
    , 'updates the content with the new toc'
  ) 
  t.end()
})

// bigbucket.org
check(
    [ '# My Module'
    , 'Some text here'
    , '## API'
    , '### Method One'
    , 'works like this'
    , '### Method Two'
    , '#### Main Usage'
    , 'some main usage here'
    ].join('\n')
  , [ '**Table of Contents**  *generated with [DocToc](http://doctoc.herokuapp.com/)*\n\n'
    , '- [My Module](#markdown-header-my-module)\n'
    ,   '\t- [API](#markdown-header-api)\n'
    ,     '\t\t- [Method One](#markdown-header-method-one)\n'
    ,     '\t\t- [Method Two](#markdown-header-method-two)\n'
    ,         '\t\t\t- [Main Usage](#markdown-header-main-usage)\n\n\n'
    ].join('')
  , 'bitbucket.org'
)
